/*
 * Copyright 2013 Magnus Hjelmblom, University of Gvle, Sweden
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */

package se.hig.estates;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Stack;

import javax.swing.Box;
import javax.swing.JApplet;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JTextField;

/**
 * @author mbm
 *
 */
public class EstatesApplet extends JApplet {
	/**
	 * 
	 */
	private static final String DEFAULT_HOST = "localhost";
	private static final String DEFAULT_PORT = "50002";
	protected String host = "localhost";
	protected int port = -1;
	protected Box centerBox = Box.createVerticalBox();
	protected Box hostBox = Box.createHorizontalBox();
	protected Box portBox = Box.createHorizontalBox();
	protected JLabel hostLabel = new JLabel("Host:");
	protected JTextField hostTxt = new JTextField(host);
	protected JLabel portLabel = new JLabel("Port:");
	protected JTextField portTxt = new JTextField("" + port);
	protected JButton startBtn = new JButton("Start New Client");
	protected Stack<EstatesFrame> roomsFrames = new Stack<EstatesFrame>();

	/* (non-Javadoc)
	 * @see java.applet.Applet#init()
	 */
	@Override
	public void init() {
		final int LW = 40;
		final int LH = 20;
		hostLabel.setPreferredSize(new Dimension(LW,LH));
		hostBox.add(hostLabel);
		hostTxt.setPreferredSize(new Dimension(LW*2,LH));
		hostBox.add(hostTxt);
		hostTxt.setText(DEFAULT_HOST);
		centerBox.add(hostBox);
		portLabel.setPreferredSize(new Dimension(LW,LH));
		portBox.add(portLabel);
		portTxt.setPreferredSize(new Dimension(LW*2,LH));
		portBox.add(portTxt);
		portTxt.setText(DEFAULT_PORT);
		centerBox.add(portBox);
		startBtn.addActionListener(new ActionListener() {			
			@Override
			public void actionPerformed(ActionEvent e) {
				startClient();
			}
		});
		centerBox.add(startBtn);
		this.add(centerBox);
		super.init();
	}
	
	@Override
	public void start() {
		startClient();
	}

	protected void startClient() {
		try {
			if("".equals(hostTxt.getText()))
				host = DEFAULT_HOST;
			else
				host = hostTxt.getText();
			port = Integer.parseInt(portTxt.getText());
			EstatesFrame newFrame = new EstatesFrame(host,port,3);
			newFrame.setVisible(true);
			newFrame.pack();
			roomsFrames.push(newFrame);
		} catch(NumberFormatException nfe) {
			JOptionPane.showMessageDialog(centerBox,nfe.getMessage(),"Error",JOptionPane.ERROR_MESSAGE);
		}				
	}

	@Override
	public void stop() {
		while(!roomsFrames.isEmpty())
			roomsFrames.pop().dispose();
	}
	
}
