/*
 * Copyright 2013 Magnus Hjelmblom, University of Gvle, Sweden
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */

package se.hig.nrts;

import java.util.LinkedList;
import java.util.List;
import java.util.Stack;

import se.sics.prologbeans.PBTerm;

public class Agent extends GenericComponent {
	protected String agentID;

	/**
	 * @param agentID
	 */
	protected Agent(String agentID) {
		this.agentID = agentID;
	}
	
	/**
	 * @return the agentID
	 */
	public String getAgentID() {
		return agentID;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return this.agentID;
	}

	@Override
	public int hashCode() {
		return toString().hashCode();
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		return this.toString().equals(obj.toString());
	}

	public static PBTerm createAgentListPrologTerm(List<Agent> agentList) {
		Stack<Agent> agentStack = new Stack<Agent>();
		for(Agent agent : agentList)
			agentStack.push(agent);
		PBTerm agentTerm = PBTerm.NIL;
		while (!agentStack.isEmpty()) {
			Agent agent = agentStack.pop();
			agentTerm = PBTerm.makeTerm(createAgentTerm(agent.agentID),agentTerm);
		}
		return agentTerm;
	}
	
	public static List<Agent> createAgentList(PBTerm agentListTerm) {
		List<Agent> agentList = new LinkedList<Agent>();
		while (!agentListTerm.isEmptyList()) {
			Agent agent = createAgent(agentListTerm.head());
			agentList.add(agent);
			agentListTerm = agentListTerm.tail();
		}
		return agentList;
	}

	public static List<Agent> createEmptyAgentList() {
		return new LinkedList<Agent>();
	}

	public static Agent createAgent(String agentID) {
		return new Agent(agentID);
	}

	public static Agent createAgent(PBTerm agentTerm) {
		return new Agent(agentTerm.getName());
	}

	public static PBTerm createAgentTerm(String agentID) {
		return PBTerm.makeAtom(agentID);
	}
}