/*
 * Copyright 2013 Magnus Hjelmblom, University of Gvle, Sweden
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */

package se.hig.nrts;

import java.io.IOException;

import se.sics.prologbeans.*;

public class PrologSessionHandler {
	private PrologSession session = null;
	private String queryString = "shutdown(no_connections)";
	
	private static PrologSessionHandler singletonSessionHandler = 
		new PrologSessionHandler();

	/**
	 * Dold konstruktor.
	 */
	private PrologSessionHandler() {}

	/**
	 * Returnerar en referens till PrologSessionHandler-objektet.
	 */
	public static PrologSessionHandler getInstance() {
		return singletonSessionHandler;
	}
	
	/**
	 * @return the Prolog session
	 */
	public PrologSession getSession() {
		return session;
	}

	public void createSession(String host, int port) {
		System.out.print("* Creating Prolog session [" + host + ":" + port + "]... ");
		this.session = new PrologSession();
		this.session.setHost(host);
		this.session.setPort(port);
		// TODO: add timeout attribute
//		this.session.setTimeout(0);
		System.out.println("Done!");		
	}

	public boolean cleanShutdown() {
		boolean shutdown = true;
		QueryAnswer queryAnswer = null;
		try {
//			System.out.print("* Connecting... ");
			this.session.connect();
//			System.out.println("Done!");
			System.out.println("?- " + queryString + ".");
			queryAnswer = this.session.executeQuery(queryString);
			System.out.println("=> " + queryAnswer);
			if ((queryAnswer == null) || queryAnswer.queryFailed())
				shutdown = false;
		} catch(IOException x) {
			x.printStackTrace();
			shutdown = false;
		} catch(IllegalCharacterSetException x) {
			x.printStackTrace();
			shutdown = false;
		}
//		System.out.print("* Disconnecting... ");
		this.session.disconnect();
//		System.out.println("Done!");
		return shutdown;
	}

}