/*
 * Copyright 2013 Magnus Hjelmblom, University of Gvle, Sweden
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */

package se.hig.nrts.client;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JTextField;

import se.hig.nrts.exceptions.NRTSException;
import se.hig.nrts.norms.StateCondition;

public class AddConsequenceFrame extends JFrame {
	private NRTSClientPanel parent;
	
	private Box centerBox = Box.createHorizontalBox();
	private Box consequenceIDBox = Box.createVerticalBox();	
	private JLabel consequenceIDLabel = new JLabel("Consequence ID");
	private JTextField consequenceIDTxt = new JTextField();
	private Box consequenceModuleBox = Box.createVerticalBox();
	private JLabel consequenceModuleLabel = new JLabel("Module");
	private JTextField consequenceModuleTxt = new JTextField();
	private Box consequenceFunctorBox = Box.createVerticalBox();	
	private JLabel consequenceFunctorLabel = new JLabel("Functor");
	private JTextField consequenceFunctorTxt = new JTextField();
	private Box consequenceAgentArityBox = Box.createVerticalBox();	
	private JLabel consequenceAgentArityLabel = new JLabel("Number of agent arguments");
	private JTextField consequenceAgentArityTxt = new JTextField();
	private Box consequenceArityBox = Box.createVerticalBox();	
	private JTextField consequenceArityTxt = new JTextField();
	private JLabel consequenceArityLabel = new JLabel("Number of parameters");
	
	private JButton addConsequenceBtn = new JButton("Add consequence");

	public AddConsequenceFrame(NRTSClientPanel parent) {
		super("Add consequence");
		this.parent = parent;
		initFrame();
	}

	private void initFrame() {
		this.setLayout(new BorderLayout());
				
		// --- CENTER BOX ---
		centerBox.add(consequenceIDBox);
		consequenceIDBox.add(consequenceIDLabel);
		consequenceIDBox.add(consequenceIDTxt);
		centerBox.add(consequenceModuleBox);
		consequenceModuleBox.add(consequenceModuleLabel);
		consequenceModuleBox.add(consequenceModuleTxt);
		consequenceModuleTxt.setText("user");
		centerBox.add(consequenceFunctorBox);		
		consequenceFunctorBox.add(consequenceFunctorLabel);
		consequenceFunctorBox.add(consequenceFunctorTxt);
		centerBox.add(consequenceAgentArityBox);				
		consequenceAgentArityBox.add(consequenceAgentArityLabel);
		consequenceAgentArityBox.add(consequenceAgentArityTxt);
		centerBox.add(consequenceArityBox);				
		consequenceArityBox.add(consequenceArityLabel);
		consequenceArityBox.add(consequenceArityTxt);
		
		// --- MAIN FRAME ---
		this.add(centerBox,BorderLayout.CENTER);
		addConsequenceBtn.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				try {
					String consequenceID = consequenceIDTxt.getText();
					String consequenceModule = consequenceModuleTxt.getText();
					String consequenceFunctor = consequenceFunctorTxt.getText();
					int consequenceAgentArity = 
						Integer.parseInt(consequenceAgentArityTxt.getText());
					int consequenceArity = Integer.parseInt(consequenceArityTxt.getText());
					StateCondition cCond = StateCondition.createStateCondition(consequenceFunctor,
							consequenceArity,consequenceAgentArity);
					parent.addConditionToConsequenceSet(consequenceID,cCond);
//					parent.addConditionToConsequenceSet(consequenceID,
//							consequenceModule,consequenceFunctor,
//							consequenceAgentArity,consequenceArity);
					parent.refreshConsequenceSet();
					AddConsequenceFrame.this.dispose();
				} catch (NumberFormatException nfe) {
					JOptionPane.showMessageDialog(AddConsequenceFrame.this, 
							nfe.getMessage(),"ERROR",JOptionPane.ERROR_MESSAGE);					
				} catch (NRTSException de) {
					JOptionPane.showMessageDialog(AddConsequenceFrame.this, 
							de.getMessage(),"ERROR",JOptionPane.ERROR_MESSAGE);					
				}
			}
		});
		this.add(addConsequenceBtn,BorderLayout.SOUTH);
		this.pack();
	}
}
