/*
 * Copyright 2013 Magnus Hjelmblom, University of Gvle, Sweden
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */

package se.hig.nrts.client;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JTextField;

import se.hig.nrts.exceptions.NRTSException;
import se.hig.nrts.norms.MoveOperator;
import se.hig.nrts.norms.SitCondition;
import se.hig.nrts.norms.StateCondition;

public class AddGroundFrame extends JFrame {
	private NRTSClientPanel parent;
	
	private Box centerBox = Box.createHorizontalBox();
	private Box groundIDBox = Box.createVerticalBox();	
	private JLabel groundIDLabel = new JLabel("Ground ID");
	private JTextField groundIDTxt = new JTextField();
	private Box groundModuleBox = Box.createVerticalBox();
	private JLabel groundModuleLabel = new JLabel("Module");
	private JTextField groundModuleTxt = new JTextField();
	private Box groundFunctorBox = Box.createVerticalBox();	
	private JLabel groundFunctorLabel = new JLabel("Functor");
	private JTextField groundFunctorTxt = new JTextField();
	private Box groundAgentArityBox = Box.createVerticalBox();	
	private JLabel groundAgentArityLabel = new JLabel("Number of agent arguments");
	private JTextField groundAgentArityTxt = new JTextField();
	private Box groundArityBox = Box.createVerticalBox();	
	private JTextField groundArityTxt = new JTextField();
	private JLabel groundArityLabel = new JLabel("Number of parameters");
	
	private JButton addGroundBtn = new JButton("Add ground");

	public AddGroundFrame(NRTSClientPanel parent) {
		super("Add ground");
		this.parent = parent;
		initFrame();
	}

	private void initFrame() {
		this.setLayout(new BorderLayout());
				
		// --- CENTER BOX ---
		centerBox.add(groundIDBox);
		groundIDBox.add(groundIDLabel);
		groundIDBox.add(groundIDTxt);
		centerBox.add(groundModuleBox);
		groundModuleBox.add(groundModuleLabel);
		groundModuleBox.add(groundModuleTxt);
		groundModuleTxt.setText("user");
		centerBox.add(groundFunctorBox);		
		groundFunctorBox.add(groundFunctorLabel);
		groundFunctorBox.add(groundFunctorTxt);
		centerBox.add(groundAgentArityBox);				
		groundAgentArityBox.add(groundAgentArityLabel);
		groundAgentArityBox.add(groundAgentArityTxt);
		centerBox.add(groundArityBox);				
		groundArityBox.add(groundArityLabel);
		groundArityBox.add(groundArityTxt);
		
		// --- MAIN FRAME ---
		this.add(centerBox,BorderLayout.CENTER);
		addGroundBtn.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				try {
					String groundID = groundIDTxt.getText();
					String groundModule = groundModuleTxt.getText();
					String groundFunctor = groundFunctorTxt.getText();
					int groundAgentArity = Integer.parseInt(groundAgentArityTxt.getText());
					int groundArity = Integer.parseInt(groundArityTxt.getText());
					StateCondition gCond = StateCondition.createStateCondition(groundModule,
							groundFunctor,groundArity,groundAgentArity);
					parent.addConditionToGroundSet(groundID,gCond);
//					parent.addConditionToGroundSet(groundID,
//							groundModule,groundFunctor,groundAgentArity,groundArity);
					parent.refreshGroundSet();
					AddGroundFrame.this.dispose();
				} catch (NumberFormatException nfe) {
					JOptionPane.showMessageDialog(AddGroundFrame.this, 
							nfe.getMessage(),"ERROR",JOptionPane.ERROR_MESSAGE);					
				} catch (NRTSException ne) {
					JOptionPane.showMessageDialog(AddGroundFrame.this, 
							ne.getMessage(),"ERROR",JOptionPane.ERROR_MESSAGE);					
				}
			}
		});
		this.add(addGroundBtn,BorderLayout.SOUTH);
		this.pack();
	}
}
