/*
 * Copyright 2013 Magnus Hjelmblom, University of Gvle, Sweden
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */

package se.hig.nrts.client;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;

import se.hig.nrts.exceptions.NRTSException;
import se.hig.nrts.norms.Consequence;
import se.hig.nrts.norms.Ground;
import se.hig.nrts.norms.MoveOperator;
import se.hig.nrts.norms.NormSystem;
import se.hig.nrts.norms.Operator;
import se.hig.nrts.norms.SitCondition;
import se.hig.nrts.norms.TypeOperator;

/**
 * @author mbm
 *
 */
public class AddNormFrame extends JFrame {
	private static int nextInt = 0;
	private final int W = 60;
	private final int H = 20;
	
	private NRTSClientPanel parent;
	
	private JLabel normIDLabel = new JLabel("Norm ID:");
	private JTextField normIDTxt = new JTextField();
	private JLabel arrowLabel = new JLabel(" --> ");
	
	private Box northBox = Box.createHorizontalBox();
	private Box centerBox = Box.createHorizontalBox();
	
	private JComboBox<Operator> groundOpBox = null;
	private JList<String> groundConditionList = null;
	private JScrollPane groundConditionScrollPane = new JScrollPane();
	private JTextField groundConditionParameterTxt = new JTextField();
	
	private JComboBox<Operator> consequenceOpBox = null;
	private JList<String> consequenceConditionList = null;
	private JScrollPane consequenceConditionScrollPane = new JScrollPane();
	private JTextField consequenceConditionParameterTxt = new JTextField();
	
	private JButton addNormBtn = new JButton("Add norm");

	public AddNormFrame(NRTSClientPanel parent) {
		super("Add elementary norm");
		this.parent = parent;
		initFrame();
	}

	private void initFrame() {
		this.setLayout(new BorderLayout());
		// --- NORTH BOX ---
		northBox.add(normIDLabel);
		normIDTxt.setText("n" + nextInt++);
		normIDTxt.setPreferredSize(new Dimension(W,H));
		northBox.add(normIDTxt);
		this.add(northBox, BorderLayout.NORTH);
		
		// --- CENTER BOX ---
		groundOpBox = new JComboBox<Operator>(MoveOperator.MOVE_OPS);
		centerBox.add(groundOpBox);
		
		groundConditionList = new JList<String>(parent.getGroundIDSet());
		groundConditionList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		groundConditionList.setSelectedIndex(0);
		groundConditionScrollPane.setViewportView(groundConditionList);
		centerBox.add(groundConditionScrollPane);
		centerBox.add(groundConditionParameterTxt);
		groundConditionParameterTxt.setPreferredSize(new Dimension(W,H));

		centerBox.add(arrowLabel);
		consequenceOpBox = new JComboBox<Operator>(TypeOperator.TYPE_OPS);
		centerBox.add(consequenceOpBox);
		consequenceConditionList = new JList<String>(parent.getConsequenceIDSet());
		consequenceConditionList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		consequenceConditionList.setSelectedIndex(0);
		consequenceConditionScrollPane.setViewportView(consequenceConditionList);
		centerBox.add(consequenceConditionScrollPane);
		centerBox.add(consequenceConditionParameterTxt);
		consequenceConditionParameterTxt.setPreferredSize(new Dimension(W,H));
		
		// --- MAIN FRAME ---		
		this.add(centerBox, BorderLayout.CENTER);
		addNormBtn.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				try {
					String normID = normIDTxt.getText();
					String groundOpName = MoveOperator.MOVE_OPS[groundOpBox.getSelectedIndex()].getOpID();
					String groundOpParam = MoveOperator.MOVE_OPS[groundOpBox.getSelectedIndex()].getParameter();
					String groundCondition = groundConditionList.getSelectedValue().toString();
					String groundConditionParamsString = groundConditionParameterTxt.getText();
					String consequenceOpName = TypeOperator.TYPE_OPS[consequenceOpBox.getSelectedIndex()].getOpID();
					String consequenceOpParam = TypeOperator.TYPE_OPS[consequenceOpBox.getSelectedIndex()].getParameter();
					String consequenceCondition = consequenceConditionList.getSelectedValue().toString();
					String consequenceConditionParamsString = consequenceConditionParameterTxt.getText();
					String [] groundConditionParameters = parent.splitParamsString(groundConditionParamsString);					
					String [] consequenceConditionParameters = parent.splitParamsString(consequenceConditionParamsString);
					NormSystem normSystem = parent.getNormSystem();
					Operator groundOp = Operator.createOperator(groundOpName,groundOpParam);
					Operator consequenceOp = 
							Operator.createOperator(consequenceOpName,consequenceOpParam);
					SitCondition gCond = SitCondition.createSitCond(groundOp,groundCondition,
							groundConditionParameters);
					SitCondition cCond = SitCondition.createSitCond(consequenceOp,
							consequenceCondition,consequenceConditionParameters);
					Ground ground = 
							Ground.createElementaryGround(gCond,normSystem.getGroundSet());
					Consequence consequence = 
						Consequence.createElementaryConsequence(cCond,normSystem.getConsequenceSet());
					parent.addNorm(normID,ground,consequence);
					parent.refreshNormSet();
					AddNormFrame.this.dispose();
				} catch (NRTSException ne) {
					JOptionPane.showMessageDialog(centerBox, 
							ne.getMessage(),"ERROR", JOptionPane.ERROR_MESSAGE);					
				}
			}
		});
		addNormBtn.setPreferredSize(new Dimension(W,H));
		this.add(addNormBtn, BorderLayout.SOUTH);
		this.pack();
	}
}
