/*
 * Copyright 2013 Magnus Hjelmblom, University of Gvle, Sweden
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */

package se.hig.nrts.client;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.util.Currency;
import java.util.List;
import java.util.Map;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;
import javax.swing.event.MouseInputAdapter;

import se.hig.nrts.Act;
import se.hig.nrts.Agent;
import se.hig.nrts.GenericComponent;
import se.hig.nrts.NRTSController;
import se.hig.nrts.NRTSSituation;
import se.hig.nrts.State;
import se.hig.nrts.exceptions.NRTSException;
import se.hig.nrts.norms.Consequence;
import se.hig.nrts.norms.Ground;
import se.hig.nrts.norms.Norm;
import se.hig.nrts.norms.NormSystem;
import se.hig.nrts.norms.Operator;
import se.hig.nrts.norms.SitCondition;
import se.hig.nrts.norms.StateCondition;
import se.sics.prologbeans.PBTerm;

public class NRTSClientPanel extends JPanel {
	/**
	 * 
	 */
	private static final long serialVersionUID = -6846284957062493379L;
	protected NRTSController controller = null;
	public final int WIDTH = 120;
	public final int HEIGHT = 80;
	protected Box centerBox = Box.createHorizontalBox();
	protected Box leftBox = Box.createVerticalBox();
	protected Box normSystemBox = Box.createVerticalBox();
	protected Box normsBtnsBox = Box.createHorizontalBox();
	protected Box agentsBox = Box.createVerticalBox();
	protected JList<Agent> agentsList = new JList<Agent>();
	protected JScrollPane agentsScrollPane = new JScrollPane();
	protected Box actsBox = Box.createVerticalBox();
	protected JList<Act> actsList = new JList<Act>();
	protected JScrollPane actsScrollPane = new JScrollPane();
	protected JLabel normsLabel = new JLabel("Norm-set");
	protected Box normsBox = Box.createVerticalBox();
	protected JList<Norm> normsList = new JList<Norm>();
	protected JScrollPane normsScrollPane = new JScrollPane();
	protected JButton addNormBtn = new JButton("New...");
	protected JButton delNormBtn = new JButton("Delete");
	protected JLabel groundsLabel = new JLabel("Ground-set");
	protected Box groundsBox = Box.createVerticalBox();
	protected JList<Map.Entry<String,StateCondition>> groundsList = new JList<Map.Entry<String,StateCondition>>();
	protected JScrollPane groundsScrollPane = new JScrollPane();
//	protected JButton addGroundBtn = new JButton("New...");
	protected JLabel consequencesLabel = new JLabel("Consequence-set");
	protected Box consequencesBox = Box.createVerticalBox();
	protected JList<Map.Entry<String,StateCondition>> consequencesList = new JList<Map.Entry<String,StateCondition>>();
	protected JScrollPane consequencesScrollPane = new JScrollPane();
//	protected JButton addConsequenceBtn = new JButton("New...");
	protected Box stateBox = Box.createVerticalBox();
	protected JScrollPane stateScrollPane = new JScrollPane();
	protected JList<PBTerm> stateList = new JList<PBTerm>();

	public NRTSClientPanel(String host, int port) {
		this(host,port,"_");
	}

	public NRTSClientPanel(String host, int port,String normSystemID) {
		createController(host,port,normSystemID);
		setCurrentState(State.createEmptyState());

		// --- LEFT BOX ---
		leftBox.setPreferredSize(new Dimension(2*WIDTH,6*HEIGHT));
		agentsList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		agentsList.addMouseListener(new MouseInputAdapter() {
			public void mouseClicked(MouseEvent me) {
				System.out.println("Selected agent: " + agentsList.getSelectedValue());
				controller.setAgentToMove(agentsList.getSelectedValue());
			}
		});
		agentsScrollPane.setViewportView(agentsList);
		agentsBox.setBorder(BorderFactory.createTitledBorder(BorderFactory.createLineBorder(Color.blue),
		"Agent-set"));
		agentsBox.add(agentsScrollPane);

		leftBox.add(agentsBox);

		actsScrollPane.setViewportView(actsList);
		actsBox.setBorder(BorderFactory.createTitledBorder(BorderFactory.createLineBorder(Color.blue),
		"Act-set"));
		actsBox.add(actsScrollPane);

		leftBox.add(actsBox);

		// --- NORM-SYSTEM BOX ---
		normSystemBox.setPreferredSize(new Dimension(3*WIDTH,6*HEIGHT));
		normsScrollPane.setViewportView(normsList);
		normsScrollPane.setColumnHeaderView(normsLabel);
		normsBox.add(normsScrollPane);
		normsBox.add(normsBtnsBox);
		normsBtnsBox.add(addNormBtn);
		normsBtnsBox.add(delNormBtn);
		normSystemBox.setBorder(BorderFactory.createTitledBorder(
				BorderFactory.createLineBorder(Color.blue),"Norm-system"));
		normSystemBox.add(normsBox);
		addNormBtn.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent ae) {
				AddNormFrame addNormFrame = new AddNormFrame(NRTSClientPanel.this);
				addNormFrame.setVisible(true);
			}
		});
		delNormBtn.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				int [] selected = normsList.getSelectedIndices();
				if(selected.length<1) {
					JOptionPane.showMessageDialog(NRTSClientPanel.this,
							"Please select a norm to delete.","Select norm",
							JOptionPane.ERROR_MESSAGE);
				}
				else {
					int reply = JOptionPane.showConfirmDialog(NRTSClientPanel.this,
						"Are you sure you want to delete the selected norm(s)?",
						"Delete?",JOptionPane.YES_NO_OPTION);
					if(reply == JOptionPane.YES_OPTION)
						for(int i : selected)
							deleteNorm(normsList.getModel().getElementAt(i).getNormID());
					}
			}
		});

		groundsScrollPane.setViewportView(groundsList);
		groundsScrollPane.setColumnHeaderView(groundsLabel);
		groundsBox.add(groundsScrollPane);
//		groundsBox.add(addGroundBtn);
		normSystemBox.add(groundsBox);
//		addGroundBtn.addActionListener(new ActionListener() {
//			public void actionPerformed(ActionEvent ae) {
//				AddGroundFrame addGroundFrame = new AddGroundFrame(NRTSClientPanel.this);
//				addGroundFrame.setVisible(true);
//			}
//		});

		consequencesScrollPane.setViewportView(consequencesList);
		consequencesScrollPane.setColumnHeaderView(consequencesLabel);
		consequencesBox.add(consequencesScrollPane);
//		consequencesBox.add(addConsequenceBtn);
		normSystemBox.add(consequencesBox);
//		addConsequenceBtn.addActionListener(new ActionListener() {
//			public void actionPerformed(ActionEvent ae) {
//				AddConsequenceFrame addConsequenceFrame = 
//					new AddConsequenceFrame(NRTSClientPanel.this);
//				addConsequenceFrame.setVisible(true);
//			}
//		});

		// --- STATE BOX ---
		stateBox.setPreferredSize(new Dimension(3*WIDTH,6*HEIGHT));
		stateBox.setBorder(BorderFactory.createTitledBorder(
				BorderFactory.createLineBorder(Color.blue),"State"));
		stateScrollPane.setViewportView(stateList);
		stateBox.add(stateScrollPane);
		stateList.setEnabled(false);

		// --- CENTER BOX ---
		centerBox.add(leftBox);
		centerBox.add(stateBox);
		centerBox.add(normSystemBox);

		// --- MAIN PANEL ---
		this.add(centerBox);
	}
	
	public void createController(String host, int port,String normSystemID) {
		controller = new NRTSController(host,port,normSystemID);
	}

	/**
	 * @return the NRTSController
	 */
	public NRTSController getController() {
		return controller;
	}
	
//	public void setButtonsEnabled(boolean b) {
//		this.addConsequenceBtn.setEnabled(b);
//		this.addGroundBtn.setEnabled(b);
//		this.addNormBtn.setEnabled(b);
//	}

	public boolean containsAgent(Agent agent) {
		return controller.containsAgent(agent);
	}
	
	public Agent addAgent(String agentID) {
		if (agentID == null || agentID.equals(""))
			throw new NRTSException("Agent ID must not be empty");
		Agent agent = Agent.createAgent(agentID);
		controller.addAgent(agent);
		refreshAgents();
		setAgentToMove(agent);
		return agent;
	}

	public List<Agent> getAgentSet() {
		return controller.getAgentSet();
	}

	public void setCurrentState(State state) {
		controller.setCurrentState(state);
		refreshState();
	}	

	public State getCurrentState() {
		return controller.getCurrentState();
	}	
	
	public void setAgentToMove(Agent agent) {
		controller.setAgentToMove(agent);
		refreshAgentToMove();
	}

	public void clearActs() {
		controller.clearActs();
	}

	public void addAct(String actID) {
		addAct(GenericComponent.MOD_NRTS,actID,null);
	}

	public void addAct(String actID, String actParamsString) {
		addAct(GenericComponent.MOD_NRTS,actID,actParamsString);		
	}

	public void addAct(String module, String actID, String actParamsString) {
		if (module == null || module.equals(""))
			throw new NRTSException("Module must not be empty");
		if (actID == null || actID.equals(""))
			throw new NRTSException("Act ID must not be empty");
		String [] actParams = null;
		if(actParamsString!=null && !actParamsString.equals(""))
			actParams = actParamsString.split(",");
		Act act = Act.createAct(module,actID,actParams);
		addAct(act);
	}

	public void addAct(Act act) {
		controller.addAct(act);
		refreshActs();
	}

	public List<Act> getActSet() {
		return controller.getActSet();
	}

	public void addNorm(String normID, Ground ground, Consequence consequence) {
		if (normID == null || normID.equals(""))
			throw new NRTSException("Norm ID must not be empty");		
		controller.addNorm(normID,ground,consequence);
		refreshNormSet();
	}

	public void deleteNorm(String normID) {
		controller.deleteNorm(normID);
		refreshNormSet();
	}

	public Map<String, StateCondition> getGroundSet() {
		return controller.getGroundSet();
	}

	public Map<String, StateCondition> getConsequenceSet() {
		return controller.getConsequenceSet();
	}

	public void addConditionToGroundSet(String groundID, StateCondition gCond) {
		controller.addConditionToGroundSet(groundID,gCond);
		refreshGroundSet();
	}

	public void addConditionToConsequenceSet(String consequenceID, StateCondition cCond) {
		controller.addConditionToConsequenceSet(consequenceID,cCond);
		refreshConsequenceSet();
	}
	
	public String [] getGroundIDSet() {
		return controller.getGroundIDSet();
	}
	
	public String [] getConsequenceIDSet() {
		return controller.getConsequenceIDSet();
	}

	public NormSystem getNormSystem() {
		return controller.getNormSystem();
	}
	
	public List<Act> getPermissibleActsForAgentToMove() {
		return getPermissibleActs(controller.getAgentToMove());
	}

	public List<Act> getPermissibleActs(Agent agent) {
		List<Act> permissibleActs = null;
		try {
			permissibleActs = controller.getPermissibleActs(agent);
		} catch(NRTSException x) {
			JOptionPane.showMessageDialog(centerBox,
					x.getMessage(),"Error",JOptionPane.ERROR_MESSAGE);
		}
		return permissibleActs;
	}
	
	public State getResultingState(Agent agent, Act act) {
		return controller.getResultingState(agent,act);
	}

	public Agent getSelectedAgent() {
		return agentsList.getSelectedValue();
	}

	public Act getSelectedAct() {
		return actsList.getSelectedValue();
	}

//	public NRTSSituation getCurrentSituation() {
//		return currentSituation;
//	}
//
	public void refreshSituation() {
		refreshAgents();
		refreshAgentToMove();
		refreshActs();
		refreshState();
		refreshNormSystem();
	}

	private void refreshAgents() {
		List<Agent> agents = controller.getAgentSet();
		agentsList.setListData(agents.toArray(new Agent[agents.size()]));
	}

	private void refreshAgentToMove() {
		agentsList.setSelectedValue(controller.getAgentToMove(),true);
	}

	private void refreshActs() {
		List<Act> acts = controller.getActSet();
		actsList.setListData(acts.toArray(new Act[acts.size()]));
	}

	private void refreshState() {
		State state = controller.getCurrentState();
		if(state.isList()) {
			List<PBTerm> items = State.createStateTermList(state);
			stateList.setListData(items.toArray(new PBTerm[items.size()]));
		}
		else
			stateList.setListData(new PBTerm [] {state.getPrologTerm()});
	}

	void refreshNormSet() {
		List<Norm> normSet = controller.getNormSet();
		normsList.setListData(normSet.toArray(new Norm[normSet.size()]));		
	}

	void refreshGroundSet() {
		Map<String,StateCondition> groundSet = controller.getGroundSet();
		groundsList.setListData(groundSet.entrySet().toArray(
				new Map.Entry[groundSet.size()]));	
	}

	void refreshConsequenceSet() {
		Map<String,StateCondition> consequenceSet = controller.getConsequenceSet();
		consequencesList.setListData(consequenceSet.entrySet().toArray(
				new Map.Entry[consequenceSet.size()]));		
	}

	private void refreshNormSystem() {
		refreshNormSet();
		refreshGroundSet();
		refreshConsequenceSet();
	}

	public String[] splitParamsString(String conditionParamsString) {
		if(conditionParamsString==null || conditionParamsString.equals(""))
			return null;
		return conditionParamsString.split(",");
	}

}