/*
 * Copyright 2013 Magnus Hjelmblom, University of Gvle, Sweden
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */

package se.hig.nrts.norms;

import se.hig.nrts.GenericComponent;
import se.hig.nrts.exceptions.NRTSException;
import se.sics.prologbeans.PBTerm;

public class Operator extends GenericComponent {
	protected String opID;
	protected String parameter;
//	protected int arity;

	/**
	 * @param opID
	 */
	protected Operator(String opID) {
		this(opID,null);		
	}

	/**
	 * @param opID
	 * @param parameter
	 */
	protected Operator(String opID, String parameter) {
		this.opID = opID;
//		this.arity = 1;
		if("".equals(parameter))
			this.parameter = null;
		else
			this.parameter = parameter;
	}

	/**
	 * @return the opID
	 */
	public String getOpID() {
		return opID;
	}

	/**
	 * @return the arity
	 */
	public int getArity() {
		return (parameter==null? 0 : 1);
	}

	/**
	 * @return the parameter
	 */
	public String getParameter() {
//		if (this.arity < 1)
//			throw new NRTSException("Error: operator " + this.opID + " has no parameters");
		return this.parameter;
	}
	
	public static Operator createOperator(String op, String opParameter) {
		if (op == null || op.equals(""))
			throw new NRTSException(op + "(" + opParameter + "): invalid operator");
		if (op.equals(MoveOperator.MOVE))
			if (opParameter == null)
				return new MoveOperator();
			else
				return new MoveOperator(opParameter);
		if (op.equals(TypeOperator.TYPE))
			return new TypeOperator(opParameter);
		return new Operator(op,opParameter);
	}

	public static PBTerm createPrologTerm(Operator op) {
		if (op.getArity() == 0)
			return PBTerm.makeAtom(op.getOpID());
		PBTerm [] args = new PBTerm[1];
		try {
			int p = Integer.parseInt(op.getParameter());
			args[0] = PBTerm.makeTerm(p);
		} catch (NumberFormatException nfe) {
			args[0] = PBTerm.makeAtom(op.getParameter());
		}
		return PBTerm.makeTerm(op.getOpID(),args);
	}

	public static Operator createOperator(PBTerm opTerm) {
		if (opTerm.isCompound()) {
			PBTerm parameterTerm = opTerm.getArgument(1);
			return createOperator(opTerm.getName(),parameterTerm.toString());
//			return new Operator(opTerm.getName(),parameterTerm.toString());
		}
		return createOperator(opTerm.getName(),null);
//		return new Operator(opTerm.getName());
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		if (parameter == null)
			return opID;
		return opID + this.parameter;
	}

}