/*
 * Copyright 2013 Magnus Hjelmblom, University of Gvle, Sweden
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */

package se.hig.nrts.norms;

import se.hig.nrts.GenericComponent;
import se.sics.prologbeans.PBTerm;

public class StateCondition extends GenericComponent {
	protected String module;
	protected String functor;
	protected int parameters;
	protected int agentArguments;

	
	/**
	 * @param module
	 * @param functor
	 * @param parameters
	 * @param agentArguments
	 */
	protected StateCondition(String module, String functor, int parameters, int agentArguments) {
		this.module = module;
		this.functor = functor;
		this.parameters = parameters;
		this.agentArguments = agentArguments;
	}

	/**
	 * @param functor
	 * @param parameters
	 * @param agentArguments
	 */
	protected StateCondition(String functor, int parameters, int agentArguments) {
		this(MOD_USER,functor,parameters,agentArguments);
	}

	
	/**
	 * @return the stateCondID
	 */
	public String getFunctor() {
		return functor;
	}

	/**
	 * @return the agentArguments
	 */
	public int getAgentArguments() {
		return agentArguments;
	}

	/**
	 * @return the parameters
	 */
	public int getParameters() {
		return parameters;
	}

	/**
	 * @return the module
	 */
	public String getModule() {
		return module;
	}

	public String toString() {
		return this.module + ":" 
		+ this.functor + "/" + this.parameters + "/" + this.agentArguments;
	}

	public static StateCondition createStateCondition(String functor, 
			int parameters, int agentArguments) {
		return new StateCondition(GenericComponent.MOD_NRTS,functor,parameters,agentArguments);
	}

	public static StateCondition createStateCondition(String module, String functor, 
			int parameters, int agentArguments) {
		return new StateCondition(module,functor,parameters,agentArguments);
	}

	public static StateCondition createStateCondition(PBTerm gcTerm, PBTerm parametersTerm, 
			PBTerm agentArgumentsTerm) {
		String module = gcTerm.getArgument(1).toString();
		String functor = gcTerm.getArgument(2).toString();
		int parameters = (int)parametersTerm.intValue();
		int agentArguments = (int)agentArgumentsTerm.intValue();
		return createStateCondition(module,functor,parameters,agentArguments);
	}
}