/*
 * Copyright 2013 Magnus Hjelmblom, University of Gvle, Sweden
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */

package se.hig.nrts.server;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.net.URI;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;

import se.hig.nrts.exceptions.NRTSException;

/**
 * @author mbm
 *
 */
public class NRTSServer extends JFrame {
	public final int W = 60;
	public final int H = 20;
	public final int defaultPort = 50002;
//	protected String sicstusPathString = 
//		System.getProperty("se.sics.prologbeans.prolog","sicstus");
	protected String sicstusPathString = 
			"\"C:/Program Files (x86)/SICStus Prolog VC9 4.2.0/bin/sicstus.exe\"";
	protected File sicstusPath = new File(sicstusPathString);
	protected String serverFileString = 
			"\"D:/Box Sync/DALMAS/NRTSSit/src/NRTSSit/NRTSSit_server.pl\"";			
	protected File serverFile = new File(serverFileString);
	protected String systemDefFileString = 
			"D:/Box Sync/DALMAS/NRTSSit/src/examples/estates/estates.pl";			
	protected File systemDefinitionFile = new File(systemDefFileString);
	protected String startupGoal = "start_nrts_server(" + defaultPort + ",'"+ systemDefFileString + "')";
//	protected SpecificDALMASGUI localClient = null;
	protected NRTSServerDaemon serverDaemon = null;
	protected String host = "localhost";
	protected int port = -1;

	protected JPanel centerPanel = new JPanel();
	protected JPanel messagePanel = new JPanel();

	protected JPanel chooseSicstusPathPanel = new JPanel();
	protected JLabel chooseSicstusPathLabel = new JLabel();
	protected JButton chooseSicstusPathBtn = 
		new JButton("SICStus executable...");
	protected JFileChooser sicstusPathChooser = new JFileChooser(sicstusPath);

	protected JPanel chooseServerFilePanel = new JPanel();
	protected JLabel chooseServerFileLabel = new JLabel();
	protected JButton chooseServerFileBtn = 
		new JButton("Server file...");
	protected JFileChooser serverFileChooser = new JFileChooser(serverFile);

	private JPanel chooseSystemDefFilePanel = new JPanel();
	protected JLabel chooseSystemDefFileLabel = new JLabel();
	private JButton chooseSystemDefFileBtn = new JButton("System definition file...");
	private JFileChooser systemDefFileChooser = new JFileChooser(systemDefinitionFile);

	protected JPanel startupGoalPanel = new JPanel();
	protected JLabel startupGoalLabel = new JLabel("Server startup goal: " + startupGoal);

	protected JPanel timeoutPanel = new JPanel();
	protected JLabel timeoutLabel = new JLabel("Timeout value (ms):");
	protected JTextField timeoutTxt = new JTextField();
	protected JButton timeoutBtn = new JButton("Apply new timeout value");

	protected JPanel serverControlPanel = new JPanel();
	protected JButton startServerBtn = new JButton("Start Prolog server");
	protected JButton stopServerBtn = new JButton("Stop Prolog server");
	protected JLabel serverStartedLabel = new JLabel();
	
//	protected JButton startLocalClientBtn = new JButton("Start local DALMAS client");


	public NRTSServer() {
		this(null);
	}

	public NRTSServer(String serverFileString) {
		if (serverFileString != null)
			this.serverFile = new File(serverFileString);
		initGUI();
	}

//	public NRTSServer(String serverFileString) {
//		this(serverFileString,null);
//	}

//	public NRTSServer(String serverFileString, SpecificDALMASGUI localClient) {
//		if (serverFileString != null)
//			this.serverFile = new File(serverFileString);
//		this.localClient = localClient;
//		initGUI();
//	}

	private void initGUI() {
		this.setPreferredSize(new Dimension(10*W,16*H));
		setTitle("NRTS Server");

		// --- CENTER PANEL ---
		centerPanel.setLayout(new BoxLayout(centerPanel,BoxLayout.Y_AXIS));
		this.add(centerPanel,BorderLayout.CENTER);

		// --- CHOOSE SICSTUS EXECUTABLE PANEL ---
		sicstusPathChooser.setDialogTitle("Choose SICStus executable");
		chooseSicstusPathBtn.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				int choice = sicstusPathChooser.showOpenDialog(centerPanel);
				if (choice == JFileChooser.APPROVE_OPTION)
					sicstusPath = sicstusPathChooser.getSelectedFile();
				chooseSicstusPathLabel.setText("" + sicstusPath);
			}
		});
		chooseSicstusPathPanel.add(chooseSicstusPathBtn);
		chooseSicstusPathLabel.setText("" + sicstusPath);
		chooseSicstusPathPanel.add(chooseSicstusPathLabel);
		centerPanel.add(chooseSicstusPathPanel);

		// --- CHOOSE SERVER FILE PANEL ---
		serverFileChooser.setDialogTitle("Choose server file");
		chooseServerFileBtn.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				int choice = serverFileChooser.showOpenDialog(centerPanel);
				if (choice == JFileChooser.APPROVE_OPTION) {
					serverFile = serverFileChooser.getSelectedFile();
					chooseServerFileLabel.setText("" + serverFile);
				}
			}
		});
		chooseServerFilePanel.add(chooseServerFileBtn);
		chooseServerFileLabel.setText("" + 
				(serverFile == null ? "-" : serverFile));
		chooseServerFilePanel.add(chooseServerFileLabel);
		centerPanel.add(chooseServerFilePanel);

		// --- CHOOSE SYSTEM DEF FILE PANEL ---
		systemDefFileChooser.setDialogTitle("Choose system defintion file");
		chooseSystemDefFileBtn.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				int choice = systemDefFileChooser.showOpenDialog(centerPanel);
				if (choice == JFileChooser.APPROVE_OPTION) {
					systemDefinitionFile = systemDefFileChooser.getSelectedFile();
					setTitle("NRTS Server [" + systemDefinitionFile.getName() + "]");
					chooseSystemDefFileLabel.setText("" + 
							systemDefinitionFile);
					URI systemDefURI = systemDefinitionFile.toURI();
					if(systemDefURI.isAbsolute()) // Remove / from beginning of absolute paths
						startupGoal = "start_nrts_server(" + defaultPort + ",'" + systemDefURI.getPath().substring(1) + "')";
					else
						startupGoal = "start_nrts_server(" + defaultPort + ",'" + systemDefURI.getPath() + "')";			
					startupGoalLabel.setText("Server startup goal: " + startupGoal);
				}
			}
		});
		chooseSystemDefFilePanel.add(chooseSystemDefFileBtn);
		chooseSystemDefFileLabel.setText("" + 
				(systemDefinitionFile == null ? "-" : systemDefinitionFile));
		chooseSystemDefFilePanel.add(chooseSystemDefFileLabel);
		centerPanel.add(chooseSystemDefFilePanel);

		// --- TIMEOUT PANEL ---
		timeoutPanel.add(timeoutLabel);
		timeoutTxt.setPreferredSize(new Dimension(W,H));
		timeoutTxt.setText("8000");
		timeoutBtn.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				applyNewTimeoutValue();
			}
		});
		timeoutPanel.add(timeoutTxt);
		timeoutPanel.add(timeoutBtn);
		centerPanel.add(timeoutPanel);

		// --- STARTUP GOAL PANEL ---
		startupGoalPanel.add(startupGoalLabel);
		centerPanel.add(startupGoalPanel);

		// --- SERVER CONTROL PANEL ---
		startServerBtn.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				if ((serverDaemon != null) && serverDaemon.isRunning())
					JOptionPane.showMessageDialog(centerPanel, 
							"The server is already running", 
							"Note",JOptionPane.INFORMATION_MESSAGE);
//				else if (serverFile == null)
//					JOptionPane.showMessageDialog(centerPanel,
//							"Please choose a NRTS server file", 
//							"Note",JOptionPane.INFORMATION_MESSAGE);
				else {
					startServer();
					chooseSicstusPathBtn.setEnabled(false);
					chooseServerFileBtn.setEnabled(false);
				}
			}
		});
		serverControlPanel.add(startServerBtn);

		stopServerBtn.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				if ((serverDaemon == null) || !serverDaemon.isRunning())
					JOptionPane.showMessageDialog(centerPanel, 
							"The server is not running", 
							"Note",JOptionPane.INFORMATION_MESSAGE);
				else {
					shutdownServer();
					chooseSicstusPathBtn.setEnabled(true);
					chooseServerFileBtn.setEnabled(true);
				}
			}
		});
		serverControlPanel.add(stopServerBtn);

//		startLocalClientBtn.setEnabled(false);
//		startLocalClientBtn.addActionListener(new ActionListener() {
//			public void actionPerformed(ActionEvent ae) {
//				if (localClient != null && serverDaemon.isRunning())
//					startLocalClient();
//				else {
//					JOptionPane.showMessageDialog(centerPanel, 
//							"Local client not available", 
//							"Note", JOptionPane.ERROR_MESSAGE);
//				}
//			}
//		});
//		serverControlPanel.add(startLocalClientBtn);
		centerPanel.add(serverControlPanel);

		// --- MESSAGE PANEL ---
		messagePanel.add(serverStartedLabel);
		this.add(messagePanel,BorderLayout.SOUTH);

		// --- MAIN FRAME ---
		this.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		this.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent we) {
				if ((serverDaemon != null) && serverDaemon.isRunning())
					shutdownServer();
				System.exit(0);
			}
		});
		this.pack();
	}

	public void applyNewTimeoutValue() {
		try {
			int timeout = Integer.parseInt(timeoutTxt.getText());
			serverDaemon.setTimeout(timeout);
		} catch (NumberFormatException nfe) {
			JOptionPane.showMessageDialog(centerPanel, 
					"Timeout value must be an integer value", 
					"Failure", JOptionPane.ERROR_MESSAGE);
		} catch (NRTSException de) {
			JOptionPane.showMessageDialog(centerPanel,de.getMessage(), 
					"Failure",JOptionPane.ERROR_MESSAGE);
		}
	}

	private void startServer() {
		serverDaemon = 
			new NRTSServerDaemon(sicstusPath,serverFile,startupGoal);
		applyNewTimeoutValue();
		serverDaemon.start();
		port = serverDaemon.getPort();
		if (port < 0)
			JOptionPane.showMessageDialog(this,"The server could not be started", 
					"Failure",JOptionPane.ERROR_MESSAGE);
		else
		{
			serverStartedLabel.setText("Server listening at port " + port);
//			startLocalClientBtn.setEnabled(true);
		}			
	}

	private void shutdownServer () {
//		startLocalClientBtn.setEnabled(false);
		serverStartedLabel.setText("Terminating server... ");
		serverDaemon.shutdown();
		port = -1;
		serverStartedLabel.setText("Server terminated");
	}

//	private void startLocalClient() {
//		if (localClient != null)
//			try {
//				localClient.startClient(host,port);
//			} catch (NTRSException de) {
//				JOptionPane.showMessageDialog(centerPanel,de.getMessage(), 
//						"Failure",JOptionPane.ERROR_MESSAGE);					
//			}
//			else
//				JOptionPane.showMessageDialog(centerPanel,"Local client not available", 
//						"Note",JOptionPane.INFORMATION_MESSAGE);			
//	}

	public static void main(String [] args) {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
//				NRTSServer serverGUI = new NRTSServer(null);			
				NRTSServer serverGUI = new NRTSServer();			
				serverGUI.setVisible(true);
			}
		});
	}

}
